@@ -1,4 +1,4 @@
-/* $Id: curl.c,v 1.30 2001/11/05 12:13:08 kjetilja Exp $ */
+/* $Id: curl.c,v 1.35 2001/12/06 11:44:12 kjetilja Exp $ */
 
 /* cURL Python module by Kjetil Jacobsen <kjetilja @ cs.uit.no> */
 
@@ -28,7 +28,9 @@ typedef struct {
     char error[CURL_ERROR_SIZE];
 } CurlObject;
 
+#if !defined(__cplusplus)
 staticforward PyTypeObject Curl_Type;
+#endif
 
 /* --------------------------------------------------------------------- */
 
@@ -93,7 +95,7 @@ do_cleanup(CurlObject *self, PyObject *args)
 
 /* --------------------------------------------------------------------- */
 
-static int 
+static int
 write_callback(void *ptr,
 	       size_t size,
 	       size_t nmemb,
@@ -103,7 +105,7 @@ write_callback(void *ptr,
     PyObject *result;
     CurlObject *self;
     int write_size;
-    
+
     self = (CurlObject *)stream;
     arglist = Py_BuildValue("(s#)", (char *)ptr, size*nmemb);
 
@@ -124,7 +126,7 @@ write_callback(void *ptr,
 }
 
 
-static int 
+static int
 header_callback(void *ptr,
 		size_t size,
 		size_t nmemb,
@@ -134,7 +136,7 @@ header_callback(void *ptr,
     PyObject *result;
     CurlObject *self;
     int write_size;
-    
+
     self = (CurlObject *)stream;
     arglist = Py_BuildValue("(s#)", (char *)ptr, size*nmemb);
 
@@ -155,7 +157,7 @@ header_callback(void *ptr,
 }
 
 
-static int 
+static int
 progress_callback(void *client,
 		  size_t dltotal,
 		  size_t dlnow,
@@ -166,7 +168,7 @@ progress_callback(void *client,
     PyObject *result;
     CurlObject *self;
     int ret;
-    
+
     self = (CurlObject *)client;
     arglist = Py_BuildValue("(iiii)", dltotal, dlnow, ultotal, ulnow);
 
@@ -189,8 +191,8 @@ progress_callback(void *client,
 
 static
 int password_callback(void *client,
-		      char *prompt, 
-		      char* buffer, 
+		      char *prompt,
+		      char* buffer,
 		      int buflen)
 {
     PyObject *arglist;
@@ -198,7 +200,7 @@ int password_callback(void *client,
     CurlObject *self;
     char *buf;
     int ret;
-    
+
     self = (CurlObject *)client;
     arglist = Py_BuildValue("(si)", prompt, buflen);
 
@@ -246,11 +248,11 @@ int read_callback(void *ptr,
     char *buf;
     int obj_size, read_size;
     int ret;
-    
+
     self = (CurlObject *)stream;
     read_size = size*nmemb;
     arglist = Py_BuildValue("(i)", read_size);
-    
+
     PyEval_AcquireThread(self->state);
     result = PyEval_CallObject(self->r_cb, arglist);
     Py_DECREF(arglist);
@@ -351,7 +353,7 @@ do_setopt(CurlObject *self, PyObject *args)
 	    strcpy(buf, stringdata);
 	    self->url = buf;
 	    res = curl_easy_setopt(self->handle, CURLOPT_URL, self->url);
-	} 
+	}
 	else {
 	    /* Handle the regular cases of string arguments */
 	    res = curl_easy_setopt(self->handle, option, stringdata);
@@ -360,7 +362,7 @@ do_setopt(CurlObject *self, PyObject *args)
 	if (res == 0) {
 	    Py_INCREF(Py_None);
 	    return Py_None;
-	} 
+	}
 	else {
 	    PyErr_SetString(ErrorObject, self->error);
 	    return NULL;
@@ -370,7 +372,7 @@ do_setopt(CurlObject *self, PyObject *args)
     PyErr_Clear();
 
     /* Handle the case of integer arguments */
-    if (PyArg_ParseTuple(args, "il:setopt", &option, &longdata)) {    
+    if (PyArg_ParseTuple(args, "il:setopt", &option, &longdata)) {
 	/* Check that option is integer as well as the input data */
 	if (option >= CURLOPTTYPE_OBJECTPOINT) {
 	    PyErr_SetString(ErrorObject, "integers are not supported for this option");
@@ -381,7 +383,7 @@ do_setopt(CurlObject *self, PyObject *args)
 	if (res == 0) {
 	    Py_INCREF(Py_None);
 	    return Py_None;
-	} 
+	}
 	else {
 	    PyErr_SetString(ErrorObject, self->error);
 	    return NULL;
@@ -393,9 +395,9 @@ do_setopt(CurlObject *self, PyObject *args)
     /* Handle the case of file objects */
     if (PyArg_ParseTuple(args, "iO!:setopt", &option, &PyFile_Type, &obj)) {
 	/* Ensure the option specified a file as well as the input */
-	if (!(option == CURLOPT_FILE || 
+	if (!(option == CURLOPT_FILE ||
 	      option == CURLOPT_INFILE ||
-	      option == CURLOPT_WRITEHEADER || 
+	      option == CURLOPT_WRITEHEADER ||
 	      option == CURLOPT_PROGRESSDATA ||
 	      option == CURLOPT_PASSWDDATA))
 	    {
@@ -412,11 +414,11 @@ do_setopt(CurlObject *self, PyObject *args)
 	if (res == 0) {
 	    Py_INCREF(Py_None);
 	    return Py_None;
-	} 
+	}
 	else {
 	    PyErr_SetString(ErrorObject, self->error);
 	    return NULL;
-	}      
+	}
     }
 
     PyErr_Clear();
@@ -425,13 +427,13 @@ do_setopt(CurlObject *self, PyObject *args)
     if (PyArg_ParseTuple(args, "iO!:setopt", &option, &PyList_Type, &obj)) {
 	switch (option) {
 	case CURLOPT_HTTPHEADER:
-	    slist = &self->httpheader; 
+	    slist = &self->httpheader;
 	    break;
 	case CURLOPT_QUOTE:
-	    slist = &self->quote; 
+	    slist = &self->quote;
 	    break;
 	case CURLOPT_POSTQUOTE:
-	    slist = &self->postquote; 
+	    slist = &self->postquote;
 	    break;
 	case CURLOPT_HTTPPOST:
 	    slist = NULL;
@@ -441,7 +443,7 @@ do_setopt(CurlObject *self, PyObject *args)
 	    PyErr_SetString(PyExc_TypeError, "lists are not supported for this option");
 	    return NULL;
 	}
-	
+
 	/* Handle HTTPPOST different since we construct a HttpPost form struct */
 	if (option == CURLOPT_HTTPPOST) {
 	    if (self->httppost != NULL) {
@@ -474,7 +476,7 @@ do_setopt(CurlObject *self, PyObject *args)
 	    if (res == 0) {
 		Py_INCREF(Py_None);
 		return Py_None;
-	    } 
+	    }
 	    else {
 		curl_formfree(self->httppost);
 		PyErr_SetString(ErrorObject, self->error);
@@ -484,7 +486,7 @@ do_setopt(CurlObject *self, PyObject *args)
 
 	/* Just to be sure we do not bug off here */
 	assert(slist != NULL);
-	
+
 	/* Handle regular list operations on the other options */
 	if (*slist != NULL) {
 	    /* Free previously allocated list */
@@ -511,7 +513,7 @@ do_setopt(CurlObject *self, PyObject *args)
 	if (res == 0) {
 	    Py_INCREF(Py_None);
 	    return Py_None;
-	} 
+	}
 	else {
 	    curl_slist_free_all(*slist);
 	    PyErr_SetString(ErrorObject, self->error);
@@ -603,11 +605,11 @@ do_perform(CurlObject *self, PyObject *args)
     Py_BEGIN_ALLOW_THREADS
     res = curl_easy_perform(self->handle);
     Py_END_ALLOW_THREADS
-    
+
     if (res == 0) {
 	Py_INCREF(Py_None);
 	return Py_None;
-    } 
+    }
     else {
 	PyErr_SetString(ErrorObject, self->error);
 	return NULL;
@@ -632,18 +634,18 @@ do_getinfo(CurlObject *self, PyObject *args)
 
     /* Parse option */
     if (PyArg_ParseTuple(args, "i:getinfo", &option)) {
-	if (option == CURLINFO_HEADER_SIZE || 
+	if (option == CURLINFO_HEADER_SIZE ||
 	    option == CURLINFO_REQUEST_SIZE ||
 	    option == CURLINFO_SSL_VERIFYRESULT ||
 	    option == CURLINFO_FILETIME ||
-	    option == CURLINFO_HTTP_CODE) 
+	    option == CURLINFO_HTTP_CODE)
 	    {
 		/* Return long as result */
 		res = curl_easy_getinfo(self->handle, option, &l_res);
 		/* Check for errors and return result */
 		if (res == 0) {
 		    return PyLong_FromLong(l_res);
-		} 
+		}
 		else {
 		    PyErr_SetString(ErrorObject, self->error);
 		    return NULL;
@@ -657,7 +659,7 @@ do_getinfo(CurlObject *self, PyObject *args)
 		/* Check for errors and return result */
 		if (res == 0) {
 		    return PyString_FromString(s_res);
-		} 
+		}
 		else {
 		    PyErr_SetString(ErrorObject, self->error);
 		    return NULL;
@@ -668,6 +670,7 @@ do_getinfo(CurlObject *self, PyObject *args)
 	    option == CURLINFO_NAMELOOKUP_TIME ||
 	    option == CURLINFO_CONNECT_TIME ||
 	    option == CURLINFO_PRETRANSFER_TIME ||
+	    option == CURLINFO_STARTTRANSFER_TIME ||
 	    option == CURLINFO_SIZE_UPLOAD ||
 	    option == CURLINFO_SIZE_DOWNLOAD ||
 	    option == CURLINFO_SPEED_DOWNLOAD ||
@@ -680,13 +683,13 @@ do_getinfo(CurlObject *self, PyObject *args)
 		/* Check for errors and return result */
 		if (res == 0) {
 		    return PyFloat_FromDouble(d_res);
-		} 
+		}
 		else {
 		    PyErr_SetString(ErrorObject, self->error);
 		    return NULL;
 		}
 	    }
-    } 
+    }
 
     /* Got wrong signature on the method call */
     PyErr_SetString(PyExc_TypeError, "invalid arguments to getinfo");
@@ -706,7 +709,7 @@ static PyMethodDef curlobject_methods[] = {
     {"setopt", (PyCFunction)do_setopt, METH_VARARGS, co_setopt_doc},
     {"perform", (PyCFunction)do_perform, METH_VARARGS, co_perform_doc},
     {"getinfo", (PyCFunction)do_getinfo, METH_VARARGS, co_getinfo_doc},
-    {NULL, NULL}
+    {NULL, NULL, 0, NULL}
 };
 
 
@@ -733,7 +736,10 @@ statichere PyTypeObject Curl_Type = {
     0,			    /*tp_as_number*/
     0,			    /*tp_as_sequence*/
     0,			    /*tp_as_mapping*/
-    0,			    /*tp_hash*/
+    0			    /*tp_hash*/
+    /* More fields follow here, depending on your Python version.
+     * You can safely ignore any compiler warnings.
+     */
 };
 
 /* --------------------------------------------------------------------- */
@@ -741,50 +747,20 @@ statichere PyTypeObject Curl_Type = {
 static CurlObject *
 do_init(PyObject *arg)
 {
-    CURL *curlhandle;
     CurlObject *self;
     int res;
 
-    /* Initialize curl */
-    curlhandle = curl_easy_init();
-    if (curlhandle == NULL) {
-	return NULL;
-    }
-
     /* Allocate python curl object */
 #if (PY_VERSION_HEX < 0x01060000)
     self = (CurlObject *) PyObject_NEW(CurlObject, &Curl_Type);
 #else
     self = (CurlObject *) PyObject_New(CurlObject, &Curl_Type);
 #endif
-    if (self == NULL) {
-	curl_easy_cleanup(curlhandle);
+    if (self == NULL)
 	return NULL;
-    }
 
-    /* Set error buffer */
-    res = curl_easy_setopt(curlhandle, CURLOPT_ERRORBUFFER, self->error);
-    if (res != 0) {
-	curl_easy_cleanup(curlhandle);
-	return NULL;
-    }
-    memset(self->error, 0, sizeof(char) * CURL_ERROR_SIZE);
-
-    /* Set NOPROGRESS to 1 by default */
-    res = curl_easy_setopt(curlhandle, CURLOPT_NOPROGRESS, 1);
-    if (res != 0) {
-	curl_easy_cleanup(curlhandle);
-	return NULL;
-    }
-    /* Set VERBOSE to 0 by default */
-    res = curl_easy_setopt(curlhandle, CURLOPT_VERBOSE, 0);
-    if (res != 0) {
-	curl_easy_cleanup(curlhandle);
-	return NULL;
-    }
-
-    /* Setup python curl object initial values and return object */
-    self->handle = curlhandle;
+    /* Setup python curl object initial values */
+    self->handle = NULL;
     self->url = NULL;
     self->httpheader = NULL;
     self->quote = NULL;
@@ -798,7 +774,33 @@ do_init(PyObject *arg)
     self->pro_cb = NULL;
     self->pwd_cb = NULL;
 
+    /* Initialize curl */
+    self->handle = curl_easy_init();
+    if (self->handle == NULL)
+        goto error;
+
+    /* Set error buffer */
+    res = curl_easy_setopt(self->handle, CURLOPT_ERRORBUFFER, self->error);
+    if (res != 0)
+        goto error;
+    memset(self->error, 0, sizeof(char) * CURL_ERROR_SIZE);
+
+    /* Set NOPROGRESS to 1 by default */
+    res = curl_easy_setopt(self->handle, CURLOPT_NOPROGRESS, 1);
+    if (res != 0)
+        goto error;
+    /* Set VERBOSE to 0 by default */
+    res = curl_easy_setopt(self->handle, CURLOPT_VERBOSE, 0);
+    if (res != 0)
+        goto error;
+
+    /* Success - return new object */
     return self;
+
+error:
+    Py_DECREF(self);    /* this also closes self->handle */
+    PyErr_SetString(ErrorObject, "initializing curl failed");
+    return NULL;
 }
 
 
@@ -822,7 +824,7 @@ do_global_init(PyObject *self, PyObject *args)
 	      option == CURL_GLOBAL_NOTHING)) {
 	    PyErr_SetString(ErrorObject, "invalid option to global_init");
 	    return NULL;
-	}	    
+	}
 
 	res = curl_global_init(option);
 	if (res != 0) {
@@ -855,7 +857,7 @@ static PyMethodDef curl_methods[] = {
     {"init", (PyCFunction)do_init, METH_VARARGS, pycurl_init_doc},
     {"global_cleanup", (PyCFunction)do_global_cleanup, METH_VARARGS, pycurl_global_cleanup_doc},
     {"global_init", (PyCFunction)do_global_init, METH_VARARGS, pycurl_global_init_doc},
-    {NULL, NULL}
+    {NULL, NULL, 0, NULL}
 };
 
 
@@ -891,15 +893,15 @@ DL_EXPORT(void)
     /* Initialize the type of the new type object here; doing it here
      * is required for portability to Windows without requiring C++. */
     Curl_Type.ob_type = &PyType_Type;
-  
+
     /* Create the module and add the functions */
     m = Py_InitModule3("pycurl", curl_methods, module_doc);
-  
+
     /* Add error object to the module */
     d = PyModule_GetDict(m);
     ErrorObject = PyErr_NewException("pycurl.error", NULL, NULL);
     PyDict_SetItemString(d, "error", ErrorObject);
-    
+
     /* Add version string to the module */
     PyDict_SetItemString(d, "version", PyString_FromString(curl_version()));
 
@@ -972,7 +974,7 @@ DL_EXPORT(void)
     insint(d, "RANDOM_FILE", CURLOPT_RANDOM_FILE);
     insint(d, "EGDSOCKET", CURLOPT_EGDSOCKET);
     insint(d, "CONNECTTIMEOUT", CURLOPT_CONNECTTIMEOUT);
-    
+
     insint(d, "HTTPGET", CURLOPT_HTTPGET);
     insint(d, "SSL_VERIFYHOST", CURLOPT_SSL_VERIFYHOST);
     insint(d, "COOKIEJAR", CURLOPT_COOKIEJAR);
@@ -980,7 +982,8 @@ DL_EXPORT(void)
     insint(d, "HTTP_VERSION", CURLOPT_HTTP_VERSION);
     insint(d, "HTTP_VERSION_1_0", CURL_HTTP_VERSION_1_0);
     insint(d, "HTTP_VERSION_1_1", CURL_HTTP_VERSION_1_1);
-    
+    insint(d, "FTP_USE_EPSV", CURLOPT_FTP_USE_EPSV);
+
     /* Symbolic constants for getinfo */
     insint(d, "EFFECTIVE_URL", CURLINFO_EFFECTIVE_URL);
     insint(d, "HTTP_CODE", CURLINFO_HTTP_CODE);
@@ -998,6 +1001,7 @@ DL_EXPORT(void)
     insint(d, "FILETIME", CURLINFO_FILETIME);
     insint(d, "CONTENT_LENGTH_DOWNLOAD", CURLINFO_CONTENT_LENGTH_DOWNLOAD);
     insint(d, "CONTENT_LENGTH_UPLOAD", CURLINFO_CONTENT_LENGTH_UPLOAD);
+    insint(d, "STARTTRANSFER_TIME", CURLINFO_STARTTRANSFER_TIME);
 
     /* CLOSEPOLICY constants for setopt */
     insint(d, "CLOSEPOLICY_LEAST_RECENTLY_USED", CURLCLOSEPOLICY_LEAST_RECENTLY_USED);
@@ -1014,3 +1018,8 @@ DL_EXPORT(void)
     /* Initialize global interpreter lock */
     PyEval_InitThreads();
 }
+
+/*
+vi:ts=8
+*/
+
